/*
 * Copyright (C) 2019-2023 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __FW_DRV_H__
#define __FW_DRV_H__

#include <linux/cdev.h>
#include <linux/semaphore.h>
#include <linux/ktime.h>

#include "../../include/fw-board.h"

#define PCI_MAJOR 0 /* dynamic major by default */
#define IRQ_TIMEOUT_MS 5000 /* 5s */
#define IRQ_VECS_MAX 4

struct fw_dev {
    struct pci_dev *pdev; /* non-null after pciprobe */
    struct device *dev;
    struct cdev cdev;
    struct fw_dev_infos infos;
    void *remap[DEVICE_COUNT_RESOURCE]; /* iomem bar remapped */
    int is_open; /* 0: free - 1: busy */
    struct list_head link; /* double linked list */
    uint32_t minor; /* minor device number (device index) */
    /* irq part */
    struct fw_dev_irq irq;
    bool has_msi_cap;
    int nb_irq_vecs;
    int it_flag;
    wait_queue_head_t it_mutex;
    spinlock_t it_lock;
    /* dma part */
    bool has_dma_cap;
    enum dma_data_direction dma_dir;
    size_t dma_bsz;
    dma_addr_t dma_addr;
    void *dma_buffer;
    bool dma_xfer_in_progress; /* transfer in progress */
};

extern struct file_operations fw_fops;

#endif // __FW_DRV_H__
