/*
 * Copyright (C) 2023 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __LIB_FIPCAP_H
#define __LIB_FIPCAP_H

#include <stddef.h> // NULL
#include <stdint.h> // uint8_t, ...

#include "fw-board.h"

#define _FIPCAP_NODE_CNT_MAX 256
#define _FIPCAP_BA_WIND_CNT_MAX 50
#define _FIPCAP_BA_WIND_REQ_CNT_MAX 1000

/**
 * ENUMERATIONS
 */
enum fipcap_sess_state {
    /* no config loaded (initial state) */
    FIPCAP_SESS_STATE_INITIAL = 0,
    /* session config loaded (stopped state: ready to start) */
    FIPCAP_SESS_STATE_READY,
    /* running state (sniffer is active) */
    FIPCAP_SESS_STATE_RUNNING,
    _FIPCAP_SESS_STATE_MAX,
};

/* error codes */
enum fipcap_error_code {
    /* to avoid to overwrite system errors */
    _FIPCAP_ERR_CODE_MIN = 256,
    /* library error codes */
    FIPCAP_ERR_DEV_ALREADY_BIND = _FIPCAP_ERR_CODE_MIN,
    FIPCAP_ERR_DEV_IRQ_HANDLER_STARTED,
    FIPCAP_ERR_DEV_IRQ_HANDLER_STOPPED,
    FIPCAP_ERR_DEV_DIAG_TASK_STARTED,
    FIPCAP_ERR_DEV_DIAG_TASK_STOPPED,
    FIPCAP_ERR_SESS_NOT_STOP,
    FIPCAP_ERR_SESS_NOT_RUN,
    FIPCAP_ERR_INVALID_CTX,
    FIPCAP_ERR_SESS_HANDLER_MISSING,
    _FIPCAP_ERR_CODE_MAX,
};

enum fipcap_frm_err_code {
    _FIPCAP_FRM_OK = 0,
    _FIPCAP_FRM_ERR_MIN,
    FIPCAP_FRM_ERR_PREAMBLE_MISSING = _FIPCAP_FRM_ERR_MIN,
    FIPCAP_FRM_ERR_FSD_MISSING,
    FIPCAP_FRM_ERR_FSD_UNKNOWN,
    FIPCAP_FRM_ERR_FED_MISSING,
    FIPCAP_FRM_ERR_CTL_UNKNOWN,
    FIPCAP_FRM_ERR_FCS_BAD,
    FIPCAP_FRM_ERR_PDU_BAD,
    FIPCAP_FRM_ERR_LEN_BAD,
    _FIPCAP_FRM_ERR_MAX,
};

enum fipcap_frm_pdu_type {
    FIPCAP_FRM_PDU_MPS = 0x40,
    FIPCAP_FRM_PDU_SMMPS = 0x50,
};

enum fipcap_frm_ctrl_type {
    FIPCAP_FRM_CTRL_ID_DAT = 0x3,
    FIPCAP_FRM_CTRL_ID_MSG = 0x5,
    FIPCAP_FRM_CTRL_ID_RQ1 = 0x29,
    FIPCAP_FRM_CTRL_ID_RQ2 = 0x9,
    FIPCAP_FRM_CTRL_RP_DAT = 0x2,
    FIPCAP_FRM_CTRL_RP_DAT_MSG = 0x6,
    FIPCAP_FRM_CTRL_RP_DAT_RQ1 = 0x2a,
    FIPCAP_FRM_CTRL_RP_DAT_RQ2 = 0xa,
    FIPCAP_FRM_CTRL_RP_DAT_RQ1_MSG = 0x2e,
    FIPCAP_FRM_CTRL_RP_DAT_RQ2_MSG = 0xe,
    FIPCAP_FRM_CTRL_RP_RQ1 = 0x28,
    FIPCAP_FRM_CTRL_RP_RQ2 = 0x8,
    FIPCAP_FRM_CTRL_RP_MSG_NOACK = 0x4,
    FIPCAP_FRM_CTRL_RP_MSG_ACK_EVEN = 0x14,
    FIPCAP_FRM_CTRL_RP_MSG_ACK_ODD = 0x94,
    FIPCAP_FRM_CTRL_RP_ACK_EVEN_P = 0x30,
    FIPCAP_FRM_CTRL_RP_ACK_ODD_P = 0xb0,
    FIPCAP_FRM_CTRL_RP_ACK_EVEN_M = 0x10,
    FIPCAP_FRM_CTRL_RP_ACK_ODD_M = 0x90,
    FIPCAP_FRM_CTRL_RP_FIN = 0x40,
};

enum fipcap_frm_usrdata_type {
    _FIPCAP_FRM_USRDATA_TYPE_MIN = 0,
    FIPCAP_FRM_USRDATA_NONE = _FIPCAP_FRM_USRDATA_TYPE_MIN,
    FIPCAP_FRM_USRDATA_VAR,
    FIPCAP_FRM_USRDATA_VAR_LIST,
    FIPCAP_FRM_USRDATA_MSG,
    _FIPCAP_FRM_USRDATA_TYPE_MAX,
    _FIPCAP_FRM_USRDATA_TYPE_UNKNOWN,
};

enum fipcap_ba_status {
    _FIPCAP_BA_STATUS_MIN,
    FIPCAP_BA_STATUS_NOT_SUPPORTED = _FIPCAP_BA_STATUS_MIN,
    FIPCAP_BA_STATUS_NOT_ELIGIBLE,
    FIPCAP_BA_STATUS_IDLE,
    FIPCAP_BA_STATUS_ACTIVE,
    _FIPCAP_BA_STATUS_MAX,
};

enum fipcap_ba_id_type {
    _FIPCAP_BA_ID_TYPE_MIN = 0,
    FIPCAP_BA_ID_UNKNOWN = _FIPCAP_BA_ID_TYPE_MIN,
    FIPCAP_BA_ID_DAT,
    FIPCAP_BA_ID_MSG,
    FIPCAP_BA_ID_RQ,
    _FIPCAP_BA_ID_TYPE_MAX,
};

enum fipcap_ba_wind_type {
    _FIPCAP_BA_WIND_TYPE_MIN = 0,
    FIPCAP_BA_WIND_UNKNOWN = _FIPCAP_BA_WIND_TYPE_MIN,
    FIPCAP_BA_WIND_PER_VAR,
    FIPCAP_BA_WIND_PER_MSG,
    FIPCAP_BA_WIND_APER_VAR,
    FIPCAP_BA_WIND_APER_MSG,
    _FIPCAP_BA_WIND_TYPE_MAX,
};

enum fipcap_bitrate {
    _FIPCAP_BITRATE_MIN = 1,
    FIPCAP_BITRATE_31K25 = _FIPCAP_BITRATE_MIN,
    FIPCAP_BITRATE_1M,
    FIPCAP_BITRATE_2M5,
    FIPCAP_BITRATE_5M,
    FIPCAP_BITRATE_12M5,
    FIPCAP_BITRATE_25M,
    _FIPCAP_BITRATE_MAX,
    _FIPCAP_BITRATE_UNKNOWN = 0,
};

enum fipcap_frm_type {
    _FIPCAP_FRM_TYPE_MIN = 1,
    FIPCAP_FRM_FIP = _FIPCAP_FRM_TYPE_MIN,
    FIPCAP_FRM_WORLDFIP,
    _FIPCAP_FRM_TYPE_MAX,
    _FIPCAP_FRM_TYPE_UNKNOWN = 0,
};

/**
 * STRUCTURES
 */
#pragma pack(1) // 1 byte alignment

/* struct: opaque */
struct fipcap_dev;

/* struct: pre-declaration */
struct fipcap_sess;

struct fipcap_date {
    union {
        uint32_t raw;
        struct {
            uint8_t day;
            uint8_t month;
            uint8_t year;
            uint8_t reserved;
        } info;
    };
};

struct fipcap_version {
    union {
        uint32_t raw;
        struct {
            uint8_t patch;
            uint8_t minor;
            uint8_t major;
            uint8_t reserved;
        } info;
    };
    /* build date */
    struct fipcap_date date;
};

struct fipcap_frm_msg_addr {
    union {
        uint32_t addr; /* data link layer address */
        struct {
            union {
                uint8_t seg; /* fip segment */
                struct {
                    /* segment number */
                    uint8_t seg_num:7;
                    /* 0: individual, 1: group */
                    uint8_t seg_group:1;
                };
            };
            union {
                uint16_t lsap; /* link service access point */
                struct {
                    /* lsap number */
                    uint16_t lsap_num:15;
                    /* 0: individual, 1: group */
                    uint16_t lsap_group:1;
                };
            };
            uint8_t reserved;
        };
    };
};

struct fipcap_frm_msg_hdr {
    struct fipcap_frm_msg_addr src; /* source address */
    struct fipcap_frm_msg_addr dst; /* destination address */
};

struct fipcap_frm_var {
    uint8_t reserved[7];
    uint8_t pdu;
    uint16_t payload_bsz;
    uint8_t *payload; /* payload frame */
};

struct fipcap_frm_var_list {
    uint32_t reserved[2];
    uint16_t ids_cnt;
    uint16_t *ids; /* ids list */
};

struct fipcap_frm_msg {
    struct fipcap_frm_msg_hdr hdr;
    uint16_t payload_bsz;
    uint8_t *payload; /* payload frame */
};

struct fipcap_frm_usrdata {
    enum fipcap_frm_usrdata_type type;
    /* content area according to user data type */
    union {
        struct fipcap_frm_var var;
        struct fipcap_frm_var_list var_list;
        struct fipcap_frm_msg msg;
    };
};

struct fipcap_frm {
    uint64_t idx;
    struct {
        uint64_t epoch;
        uint32_t frame_nstime; /* frame duration (accuracy: 100ns) */
        uint32_t interframe_nstime; /* previous inter frame duration (accuracy: 100ns) */
    } time_info;
    uint8_t error; /* enum fipcap_frm_err_code */
    uint16_t raw_bsz; /* raw data byte size */
    uint8_t *raw; /* raw data frame */
    /* dissected frame */
    uint8_t ctrl;  /* enum fipcap_frm_ctrl_type */
    uint16_t id; /* fip identifier context */
    uint16_t fcs; /* frame check sequence */
    struct fipcap_frm_usrdata usrdata; /* user data (application layer) */
};

struct fipcap_sess_status {
    uint16_t state; /* enum fipcap_sess_state */
    /* network information */
    uint8_t frm_type; /* fip frame type (see enum fipcap_frm_type) */
    uint8_t frm_bitrate; /* fip bitrate (see enum fipcap_bitrate) */
    /* capturing time */
    uint64_t start_epoch;
    uint64_t stop_epoch;
    uint64_t duration_us;
    /* fip time */
    uint32_t turn_around_ustime;
    uint32_t silence_ustime;
};

struct fipcap_sess_stats {
    /* frame counters */
    struct {
        uint64_t total;
        uint64_t id_dat;
        uint64_t id_msg;
        uint64_t id_rq1;
        uint64_t id_rq2;
        uint64_t rp_dat;
        uint64_t rp_dat_msg;
        uint64_t rp_dat_rq1;
        uint64_t rp_dat_rq2;
        uint64_t rp_dat_rq1_msg;
        uint64_t rp_dat_rq2_msg;
        uint64_t rp_rq1;
        uint64_t rp_rq2;
        uint64_t rp_msg_noack;
        uint64_t rp_msg_ack_even;
        uint64_t rp_msg_ack_odd;
        uint64_t rp_ack_even_p;
        uint64_t rp_ack_odd_p;
        uint64_t rp_ack_even_m;
        uint64_t rp_ack_odd_m;
        uint64_t rp_fin;
        uint64_t padding;
    } frm;
    /* frame error counters */
    struct {
        uint64_t total;
        /**
         * Physical layer errors
         */
        /* -> Physical Layer errors */
        /* missing preamble (glitches on line) */
        uint64_t pre_mis;
        /* missing Frame Start Delimiter (FSD) */
        uint64_t fsd_mis;
        /* unknown Frame Start Delimiter (FSD) */
        uint64_t fsd_unk;
        /* missing Frame End Delimiter (FED) */
        uint64_t fed_mis;
        /**
         * Data-Link  layer errors
         */
        /* -> Data-Link Layer errors */
        /* unknown FIP Control (CTL) */
        uint64_t ctl_unk;
        /* wrong Frame Check Sequence (FCS) */
        uint64_t fcs_bad;
        /**
         * Application layer errors
         */
        /* unknown FIP Protocol Data Unit (PDU) */
        uint64_t pdu_bad;
        /* bad FIP frame length (LEN) */
        uint64_t len_bad;
        /**
         * Library errors
         */
        /**
         * continuity break in the frame flow sent by the coprocessor.
         *
         * note: maybe the operating system is overloaded and does not
         * unpack fast enough.
         */
        uint64_t idx_loss;
    } frm_err;
};

struct fipcap_ba_req {
    uint16_t type;  /* enum fipcap_ba_id_type */
    uint16_t id; /* identifier value */
};
#define FIPCAP_BA_REQ_BSZ (sizeof(struct fipcap_ba_req))

struct fipcap_ba_per_wind {
    /* number of periodic requests */
    int req_cnt;
    /* list of periodic requests */
    struct fipcap_ba_req req_list[_FIPCAP_BA_WIND_REQ_CNT_MAX];
};

struct fipcap_ba_aper_wind {
    /* end of aperiodic window in us (relative to macrocycle start) */
    uint32_t end_ustime;
};

struct fipcap_ba_wind {
    int type; /* enum fipcap_ba_wind_type */
    union {
        /* periodic var/msg window */
        struct fipcap_ba_per_wind per;
        /* aperiodic var/msg window */
        struct fipcap_ba_aper_wind aper;
    };
};

struct fipcap_ba_mcycle {
    /* number of windows inside the macrocycle */
    int wind_cnt;
    /* list of windows */
    struct fipcap_ba_wind wind_list[_FIPCAP_BA_WIND_CNT_MAX];
    /* macrocycle duration in us */
    uint32_t duration_ustime;
};

struct fipcap_sess_node_info {
    /**
     * Presence infos
     * note: updated via 0x9002 and 0x14XY identifiers
     */
    int is_present; /* 0: absent, 1: present */
    uint64_t last_pres_epoch; /* epoch of last presence detection */
    uint64_t last_abs_epoch; /* epoch of last absence detection */
    int absence_cnt; /* absence detection counter */
    int ba_status; /* enum fipcap_ba_status */
    int ba_prio; /* [0..15] (0: highest prio) */
    /**
     * Identification
     * note: updated via 0x10XY identifier
     */
    char manufacturer_name[128];
    char model_name[128];
    uint8_t revision; /* ex: 0x23 (v2.3) */
    char tag_name[128];
    uint8_t smmps_class;
    char vendor_field[128];
    /**
     * Report
     * note: updated via 0x11XY identifier
     */
    uint16_t rx_ok_ch1;
    uint16_t rx_ok_ch2;
    uint16_t rx_nok_ch1;
    uint16_t rx_nok_ch2;
    union channel_status {
        uint16_t value;
        struct {
            uint16_t tx_quality_ch1:1;
            uint16_t tx_quality_ch2:1;
            uint16_t rx_quality_ch1:1;
            uint16_t rx_quality_ch2:1;
            uint16_t valid_ch1:1;
            uint16_t valid_ch2:1;
            uint16_t traffic_ch1:1;
            uint16_t traffic_ch2:1;
            uint16_t summary_ch1:1;
            uint16_t summary_ch2:1;
            uint16_t reserved:6;
        };
    } channel_status;
};

struct fipcap_sess_ba_info {
    /**
     * Bus Arbiter infos
     * note: updated via 0x9003 identifier
     */
    uint8_t addr; /* node address of bus arbiter */
    uint8_t mcycle_idx; /* macrocycle index in progress */
    /**
     * Macrocycle auto-detection
     */
    struct fipcap_ba_mcycle mcycle; /* sniffed macrocycle */
};

struct fipcap_sess_report {
    /* session status */
    struct fipcap_sess_status status;
    /* frame stats */
    struct fipcap_sess_stats stats;
    /* fip nodes infos */
    /* note: tab index is FIP node address */
    struct fipcap_sess_node_info node[_FIPCAP_NODE_CNT_MAX];
    /* bus arbiter infos */
    struct fipcap_sess_ba_info ba;
};

/* struct: configuration */
struct fipcap_sess_cfg {
    /**
     * General FIP settings
     */
    uint16_t padding; /* padding identifier (if set to 0: 0x9080 default value) */
    /**
     * Hardware access
     * Two methods are provided to bind the FIPWatcher component with the library:
     *   1- attach a pci/pcie device (from fipcap_dev structure)
     *      note: in this case no need to configure (dpm_base_addr)
     *   2- manually configure dual-port memory address (DPM) to access
     *      directly to the FIPWatcher chip (dpm_base_addr)
     */
    struct fipcap_dev *dev; /* pci device to bind with capture */
    unsigned long dpm_base_addr; /* dual-port memory area start address */
    /**
     * General Handlers
     */
    void *user_ctx; /* user context pointer [optional] */
    void (* fipcap_frm_handler)(struct fipcap_sess *sess,
                struct fipcap_frm *frm);
    /**
     * fipcap_diag_handler:
     * Dedicated handler to get the session report at regular intervals
     *      @31.25Kbps  : 1.6s
     *      @1Mbps      : 500ms
     *      @2.5Mbps    : 200ms
     *      @5Mbps      : 100ms
     *      @12.5Mbps   : 40ms
     *      @25Mbps     : 20ms
     */
    void (* fipcap_diag_handler)(struct fipcap_sess *sess,
                struct fipcap_sess_report *report);
    void (* fipcap_error_handler)(struct fipcap_sess *sess, /* mandatory */
                enum fipcap_error_code error);
    void (* fipcap_reset_handler)(struct fipcap_sess *sess); /* mandatory */
};

struct fipcap_sess_infos {
    /* user configuration about capture */
    struct fipcap_sess_cfg cfg;
    uint32_t sn;
    /**
     * Versions
     */
    /* driver version */
    struct fipcap_version drv_version;
    /* library version */
    struct fipcap_version lib_version;
};

/* struct: op */
struct fipcap_sess {
    /* capture informations */
    struct fipcap_sess_infos *infos;
    /* reserved opaque structure */
    void *priv;
};

#pragma pack()

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_WIN32) || defined(_WIN64)
#   if defined(FIPCAP_API_STATIC)
#       define FIPCAP_API
#   else
#       if defined(FIPCAP_API_SHARED_EXPORT)
#           define FIPCAP_API __declspec(dllexport)
#       else
#           define FIPCAP_API __declspec(dllimport)
#       endif
#   endif
#else
#   define FIPCAP_API
#endif

/**
 * FUNCTIONS
 */
/* library: init/exit, infos, errors */
FIPCAP_API int fipcap_init();
FIPCAP_API void fipcap_exit();
FIPCAP_API const struct fipcap_version *fipcap_version_get();
FIPCAP_API int fipcap_error_get();
FIPCAP_API const char *fipcap_strerror(int err);

/* device */
/* note: section reserved to pci/pcie devices */
FIPCAP_API int fipcap_device_list_get(
                struct fw_dev_infos *dev_infos, int *dev_cnt);
FIPCAP_API struct fipcap_dev *fipcap_device_open(uint8_t dev_id);
FIPCAP_API int fipcap_device_reset(struct fipcap_dev *dev);
FIPCAP_API int fipcap_device_close(struct fipcap_dev *dev);
FIPCAP_API int fipcap_device_infos_get(struct fipcap_dev *dev,
                struct fw_dev_infos *info);

/* capture session */
FIPCAP_API struct fipcap_sess *fipcap_sess_init(struct fipcap_sess_cfg *cfg);
FIPCAP_API int fipcap_sess_exit(struct fipcap_sess *sess);
FIPCAP_API int fipcap_sess_start(struct fipcap_sess *sess);
FIPCAP_API int fipcap_sess_stop(struct fipcap_sess *sess);
FIPCAP_API int fipcap_sess_report_get(struct fipcap_sess *sess,
                struct fipcap_sess_report *report);
FIPCAP_API int fipcap_sess_stats_clear(struct fipcap_sess *sess);

#ifdef __cplusplus
}
#endif

#endif /* __LIB_FIPCAP_H */
