#!/bin/sh
library="libfipcap"

FIPCAP_HDR_DIR=/usr/local/include/fipcapture
FIPCAP_LIB_DIR=/usr/local/lib

ROOT_DIR=$(pwd)
MACHINE=$(uname -m)

# force i386 machine if not x86_64
if [ "${MACHINE}" != "x86_64" ];
then
    MACHINE=i386
fi

# create the header (*.h) directory if it does not exist
if [ ! -d "${FIPCAP_HDR_DIR}" ];
then
    mkdir ${FIPCAP_HDR_DIR}
fi

# copy the static/shared library to the target directory
cp ./lib/${MACHINE}/static/${library}.a ${FIPCAP_LIB_DIR}
cp ./lib/${MACHINE}/${library}.so.* ${FIPCAP_LIB_DIR}

# copy the headers to the target directory
cp ./include/libfipcap.h ${FIPCAP_HDR_DIR}
cp ./include/fw-board.h ${FIPCAP_HDR_DIR}

# extract lib version and create shared library symbolic links
cd ${ROOT_DIR}/lib/${MACHINE}
if [ `ls | grep -Po '(?<=libfipcap.so.)\d.\d.\d' | wc -l` -eq 1 ];
then
    FIPCAP_LIB_VER="$(ls | grep -Po '(?<=libfipcap.so.)\d.\d.\d')"
    major=`echo ${FIPCAP_LIB_VER} | cut -d. -f1`
    minor=`echo ${FIPCAP_LIB_VER} | cut -d. -f2`
    patch=`echo ${FIPCAP_LIB_VER} | cut -d. -f3`
    # create symbolic links
    cd ${FIPCAP_LIB_DIR}
    ln -s ${library}.so.${major}.${minor}.${patch} ${library}.so.${major}
    ln -s ${library}.so.${major} ${library}.so
fi
