/*
 * Copyright (C) 2023 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __SESSION_H__
#define __SESSION_H__

#include "common.h"
#include "libfipcap.h"

static const char *frm_err_str[_FIPCAP_FRM_ERR_MAX] = {
    /* _FIPCAP_FRM_OK */
    "no error",
    /* FIPCAP_FRM_ERR_PREAMBLE_MISSING */
    "preamble missing",
    /* FIPCAP_FRM_ERR_FSD_MISSING */
    "frame start delimiter missing",
    /* FIPCAP_FRM_ERR_FSD_UNKNOWN */
    "frame start delimiter unknown",
    /* FIPCAP_FRM_ERR_FED_MISSING */
    "frame end delimiter missing",
    /* FIPCAP_FRM_ERR_CTL_UNKNOWN */
    "frame control unknown",
    /* FIPCAP_FRM_ERR_FCS_BAD */
    "frame check sequence is bad",
    /* FIPCAP_FRM_ERR_PDU_BAD */
    "unknown PDU",
    /* FIPCAP_FRM_ERR_LEN_BAD */
    "frame length is bad"
};

static const char *ba_status_str[_FIPCAP_BA_STATUS_MAX] = {
    /* FIPCAP_BA_STATUS_NOT_SUPPORTED */
    "not supported",
    /* FIPCAP_BA_STATUS_NOT_ELIGIBLE */
    "not eligible",
    /* FIPCAP_BA_STATUS_IDLE */
    "idle",
    /* FIPCAP_BA_STATUS_ACTIVE */
    "active"
};

static const char *ba_id_type_str[_FIPCAP_BA_ID_TYPE_MAX] = {
    /* FIPCAP_BA_ID_UNKNOWN */
    "unknown",
    /* FIPCAP_BA_ID_DAT */
    "id_dat",
    /* FIPCAP_BA_ID_MSG */
    "id_msg",
    /* FIPCAP_BA_ID_RQ */
    "id_rq"
};

static const char *ba_wind_type_str[_FIPCAP_BA_WIND_TYPE_MAX] = {
    /* FIPCAP_BA_WIND_UNKNOWN */
    "unknown",
    /* FIPCAP_BA_WIND_PER_VAR */
    "periodic var.",
    /* FIPCAP_BA_WIND_PER_MSG */
    "periodic msg.",
    /* FIPCAP_BA_WIND_APER_VAR */
    "aperiodic var.",
    /* FIPCAP_BA_WIND_APER_MSG */
    "aperiodic msg."
};

static const char *frm_bitrate_str[_FIPCAP_BITRATE_MAX] = {
    /* _FIPCAP_BITRATE_UNKNOWN */
    "unknown",
    /* FIPCAP_BITRATE_31K25 */
    "31.25kbps",
    /* FIPCAP_BITRATE_1M */
    "1mbps",
    /* FIPCAP_BITRATE_2M5 */
    "2.5mbps",
    /* FIPCAP_BITRATE_5M */
    "5mbps",
    /* FIPCAP_BITRATE_12M5 */
    "12.5mbps",
    /* FIPCAP_BITRATE_25M */
    "25mbps"
};

static const char *frm_type_str[_FIPCAP_FRM_TYPE_MAX] = {
    /* _FIPCAP_FRM_TYPE_UNKNOWN */
    "unknown",
    /* FIPCAP_FRM_FIP */
    "fip",
    /* FIPCAP_FRM_WORLDFIP */
    "worldfip"
};

void tst_fipcap_error_handler(struct fipcap_sess *cap_s,
        enum fipcap_error_code code);
void tst_fipcap_reset_handler(struct fipcap_sess *cap_s);
void tst_fipcap_frm_handler(struct fipcap_sess *cap_s,
        struct fipcap_frm *frm);
void tst_fipcap_diag_handler(struct fipcap_sess *sess,
        struct fipcap_sess_report *r);
void print_cap_infos(struct fipcap_sess *cap_s);

#endif
